import sys
import time
import datetime
import pylights3
from suds.client import Client

pylights3.device_cfg_filename='devices.xml'

# Initialize PowerLinc Modem
p = pylights3.plm()

# Fountain status
fountainRunning = False
result = p.getLevel('outdoorModule')
if result == 100:
    fountainRunning = True

# Low Water flag
lowWater = False
result = p.getSensorState('ioLinc')
if result == 1:
    lowWater = True
elif result != 0:
    # catch time out
    print('Failed to retrieve water level sensor data.')   

# Float Position Flag
floatUpper = False
result = p.getLevel('ioLinc')
if result == 100:
    floatUpper = True
elif result != 0:
    # catch time out
    print('Failed to retrieve position of water level sensor.') 

# Minute counter
minuteCounter = datetime.datetime.now().minute


def loop():
    # Run every 5 seconds
    global lowWater
    global floatUpper
    global fountainRunning
    global minuteCounter

    d = datetime.datetime.now()
    
    # Check if business hours 7am to 5pm
    businessHours = False
    if d.isoweekday() in range (1, 6) and d.hour in range (7, 17):
        businessHours = True

    # Check if float position is out of sync with low water flag (should be lower)
    if floatUpper and not lowWater:
        # Sync position flag
        result = p.setLevel('ioLinc', 0)
        if result == 'OK':
            floatUpper = False
        else:
            # catch time out
            print('Failed to switch float position to lower.')

    # Check if float position is out of sync with low water flag (should be upper)
    elif not floatUpper and lowWater:
        # Sync position flag
        result = p.setLevel('ioLinc', 100)
        if result == 'OK':
            floatUpper = True
        else:
            # catch time out
            print('Failed to switch float position to upper.')
        
    # Check if fountain should be on
    elif not fountainRunning and businessHours and not lowWater:
        result = p.setLevel('outdoorModule', 100)
        if result == 'OK':
            fountainRunning = True
        else:
            # catch time out
            print('Failed to turn fountain on.')
        
    # Check if fountain should be off
    elif fountainRunning and (not businessHours or lowWater):
        result = p.setLevel('outdoorModule', 0)
        if result == 'OK':
            fountainRunning = False
        else:
            # catch time out
            print('Failed to turn fountain off.')       
        

    # Check water level every few minutes
    elif minuteCounter != datetime.datetime.now().minute:
        minuteCounter = datetime.datetime.now().minute
        # Check for fill if low water and business hours
        if lowWater and businessHours:
            sensorState = p.getSensorState('ioLinc')
            if sensorState == 0:
                # Water refilled
                lowWater = False
                # Switch float position to lower
                result = p.setLevel('ioLinc', 0)
                if result == 'OK':
                    floatUpper = False
                else:
                    # catch time out
                    print('Failed to switch float position to lower.')
            elif sensorState != 1:
                # catch time out
                print('Failed to retrieve water level sensor data.')
        # Check if water is low every 10 minutes
        elif not lowWater and d.minute % 10 == 0:
            sensorState = p.getSensorState('ioLinc')
            if sensorState == 1:
                lowWater = True
                # Send notification via web service
                # client = Client('http://192.168.0.100/Pi/PiLog.svc?wsdl')
                # sResult = client.service.FountainNotify('jsmith')
                print('Notification: ' + sResult)
                # Switch float position to upper
                result = p.setLevel('ioLinc', 100)
                if result == 'OK':
                    floatUpper = True
                else:
                    # catch time out
                    print('Failed to switch float position to upper.')
            elif sensorState == 0:
                # Water level OK
                print('Water level OK')
            else:
                # catch time out
                print('Failed to retrieve water level sensor data.')   
    print('Low Water: {0}' .format(lowWater))
    print('Business Hours: {0}' .format(businessHours))
    print('Fountain Running {0}' .format(fountainRunning))
    print(" . . .")


def destroy():
    global p
    p.close()
    print ('Destroy executed.')




try:
    while True:
        # DO THINGS
        loop()
        time.sleep(5)
except:
    e = sys.exc_info()[0]
    print( 'Error: {0}' .format(e) )
finally:
    destroy()

        
    
        
